@@ -1,6 +1,6 @@
-/* $Id: terminal-app.c 124 2005-01-31 20:41:46Z bmeurer $ */
+/* $Id$ */
 /*-
- * Copyright (c) 2004 os-cillation e.K.
+ * Copyright (c) 2004-2008 os-cillation e.K.
  *
  * Written by Benedikt Meurer <benny@xfce.org>.
  *
@@ -41,15 +41,15 @@
 #include <terminal/terminal-app.h>
 #include <terminal/terminal-config.h>
 #include <terminal/terminal-preferences.h>
+#include <terminal/terminal-private.h>
 #include <terminal/terminal-window.h>
 
 
 
-static void               terminal_app_class_init               (TerminalAppClass   *klass);
-static void               terminal_app_init                     (TerminalApp        *app);
 static void               terminal_app_finalize                 (GObject            *object);
 static void               terminal_app_update_accels            (TerminalApp        *app);
-static GtkWidget           *terminal_app_create_window          (TerminalApp        *app,
+static void               terminal_app_update_mnemonics         (TerminalApp        *app);
+static GtkWidget         *terminal_app_create_window            (TerminalApp        *app,
                                                                  gboolean            fullscreen,
                                                                  TerminalVisibility  menubar,
                                                                  TerminalVisibility  borders,
@@ -59,6 +59,8 @@ static void               terminal_app_new_window               (TerminalWindow
                                                                  TerminalApp        *app);
 static void               terminal_app_new_window_with_terminal (TerminalWindow     *window,
                                                                  TerminalScreen     *terminal,
+                                                                 gint                x,
+                                                                 gint                y,
                                                                  TerminalApp        *app);
 static void               terminal_app_window_destroyed         (GtkWidget          *window,
                                                                  TerminalApp        *app);
@@ -80,15 +82,11 @@ struct _TerminalApp
   TerminalAccelMap    *accel_map;
   ExoXsessionClient   *session_client;
   gchar               *initial_menu_bar_accel;
-  GList               *windows;
+  GSList              *windows;
 };
 
 
 
-static GObjectClass *parent_class;
-
-
-
 GQuark
 terminal_error_quark (void)
 {
@@ -100,7 +98,7 @@ terminal_error_quark (void)
 
 
 
-G_DEFINE_TYPE (TerminalApp, terminal_app, G_TYPE_OBJECT);
+G_DEFINE_TYPE (TerminalApp, terminal_app, G_TYPE_OBJECT)
 
 
 
@@ -108,8 +106,6 @@ static void
 terminal_app_class_init (TerminalAppClass *klass)
 {
   GObjectClass *gobject_class;
-  
-  parent_class = g_type_class_peek_parent (klass);
 
   gobject_class = G_OBJECT_CLASS (klass);
   gobject_class->finalize = terminal_app_finalize;
@@ -123,6 +119,8 @@ terminal_app_init (TerminalApp *app)
   app->preferences = terminal_preferences_get ();
   g_signal_connect_swapped (G_OBJECT (app->preferences), "notify::shortcuts-no-menukey",
                             G_CALLBACK (terminal_app_update_accels), app);
+  g_signal_connect_swapped (G_OBJECT (app->preferences), "notify::shortcuts-no-mnemonics",
+                            G_CALLBACK (terminal_app_update_mnemonics), app);
 
   /* remember the original menu bar accel */
   g_object_get (G_OBJECT (gtk_settings_get_default ()),
@@ -130,9 +128,10 @@ terminal_app_init (TerminalApp *app)
                 NULL);
 
   terminal_app_update_accels (app);
+  terminal_app_update_mnemonics (app);
 
   /* connect the accel map */
-  app->accel_map = terminal_accel_map_new ();
+  app->accel_map = g_object_new (TERMINAL_TYPE_ACCEL_MAP, NULL);
 }
 
 
@@ -141,7 +140,7 @@ static void
 terminal_app_finalize (GObject *object)
 {
   TerminalApp *app = TERMINAL_APP (object);
-  GList       *lp;
+  GSList      *lp;
 
   for (lp = app->windows; lp != NULL; lp = lp->next)
     {
@@ -149,11 +148,10 @@ terminal_app_finalize (GObject *object)
       g_signal_handlers_disconnect_by_func (G_OBJECT (lp->data), G_CALLBACK (terminal_app_new_window), app);
       gtk_widget_destroy (GTK_WIDGET (lp->data));
     }
-  g_list_free (app->windows);
+  g_slist_free (app->windows);
 
-  g_signal_handlers_disconnect_by_func (G_OBJECT (app->preferences),
-                                        G_CALLBACK (terminal_app_update_accels),
-                                        app);
+  g_signal_handlers_disconnect_by_func (G_OBJECT (app->preferences), G_CALLBACK (terminal_app_update_accels), app);
+  g_signal_handlers_disconnect_by_func (G_OBJECT (app->preferences), G_CALLBACK (terminal_app_update_mnemonics), app);
   g_object_unref (G_OBJECT (app->preferences));
 
   if (app->initial_menu_bar_accel != NULL)
@@ -164,7 +162,7 @@ terminal_app_finalize (GObject *object)
 
   g_object_unref (G_OBJECT (app->accel_map));
 
-  G_OBJECT_CLASS (parent_class)->finalize (object);
+  (*G_OBJECT_CLASS (terminal_app_parent_class)->finalize) (object);
 }
 
 
@@ -175,7 +173,7 @@ terminal_app_update_accels (TerminalApp *app)
   const gchar *accel;
   gboolean     shortcuts_no_menukey;
 
-  g_object_get (G_OBJECT (app->preferences), 
+  g_object_get (G_OBJECT (app->preferences),
                 "shortcuts-no-menukey", &shortcuts_no_menukey,
                 NULL);
 
@@ -191,6 +189,21 @@ terminal_app_update_accels (TerminalApp *app)
 
 
 
+static void
+terminal_app_update_mnemonics (TerminalApp *app)
+{
+  gboolean no_mnemonics;
+
+  g_object_get (G_OBJECT (app->preferences),
+                "shortcuts-no-mnemonics", &no_mnemonics,
+                NULL);
+  g_object_set (G_OBJECT (gtk_settings_get_default ()),
+                "gtk-enable-mnemonics", !no_mnemonics,
+                NULL);
+}
+
+
+
 static GtkWidget*
 terminal_app_create_window (TerminalApp       *app,
                             gboolean           fullscreen,
@@ -207,7 +220,7 @@ terminal_app_create_window (TerminalApp       *app,
                     G_CALLBACK (terminal_app_new_window), app);
   g_signal_connect (G_OBJECT (window), "new-window-with-screen",
                     G_CALLBACK (terminal_app_new_window_with_terminal), app);
-  app->windows = g_list_append (app->windows, window);
+  app->windows = g_slist_prepend (app->windows, window);
 
   return window;
 }
@@ -221,7 +234,10 @@ terminal_app_new_window (TerminalWindow *window,
 {
   TerminalWindowAttr *win_attr;
   TerminalTabAttr    *tab_attr;
+  TerminalScreen     *terminal;
   GdkScreen          *screen;
+  gboolean            inherit_geometry;
+  gint                w, h;
 
   screen = gtk_window_get_screen (GTK_WINDOW (window));
 
@@ -230,6 +246,21 @@ terminal_app_new_window (TerminalWindow *window,
   tab_attr = win_attr->tabs->data;
   tab_attr->directory = g_strdup (working_directory);
 
+  /* check if we should try to inherit the parent geometry */
+  g_object_get (G_OBJECT (app->preferences), "misc-inherit-geometry", &inherit_geometry, NULL);
+  if (G_UNLIKELY (inherit_geometry))
+    {
+      /* determine the currently active terminal screen for the window */
+      terminal = terminal_window_get_active (window);
+      if (G_LIKELY (terminal != NULL))
+        {
+          /* let the new window inherit the geometry from its parent */
+          g_free (win_attr->geometry);
+          terminal_screen_get_size (terminal, &w, &h);
+          win_attr->geometry = g_strdup_printf ("%dx%d", w, h);
+        }
+    }
+
   terminal_app_open_window (app, win_attr);
 
   terminal_window_attr_free (win_attr);
@@ -240,16 +271,26 @@ terminal_app_new_window (TerminalWindow *window,
 static void
 terminal_app_new_window_with_terminal (TerminalWindow *existing,
                                        TerminalScreen *terminal,
+                                       gint            x,
+                                       gint            y,
                                        TerminalApp    *app)
 {
   GtkWidget *window;
   GdkScreen *screen;
 
+  terminal_return_if_fail (TERMINAL_IS_WINDOW (existing));
+  terminal_return_if_fail (TERMINAL_IS_SCREEN (terminal));
+  terminal_return_if_fail (TERMINAL_IS_APP (app));
+
   window = terminal_app_create_window (app, FALSE,
                                        TERMINAL_VISIBILITY_DEFAULT,
                                        TERMINAL_VISIBILITY_DEFAULT,
                                        TERMINAL_VISIBILITY_DEFAULT);
 
+  /* set new window position */
+  if (x > -1 && y > -1)
+    gtk_window_move (GTK_WINDOW (window), x, y);
+
   /* place the new window on the same screen as
    * the existing window.
    */
@@ -273,9 +314,9 @@ static void
 terminal_app_window_destroyed (GtkWidget   *window,
                                TerminalApp *app)
 {
-  g_return_if_fail (g_list_find (app->windows, window) != NULL);
+  terminal_return_if_fail (g_slist_find (app->windows, window) != NULL);
 
-  app->windows = g_list_remove (app->windows, window);
+  app->windows = g_slist_remove (app->windows, window);
 
   if (G_UNLIKELY (app->windows == NULL))
     gtk_main_quit ();
@@ -287,21 +328,21 @@ static void
 terminal_app_save_yourself (ExoXsessionClient *client,
                             TerminalApp       *app)
 {
-  GList               *result = NULL;
-  GList               *lp;
-  gchar              **oargv;
-  gchar              **argv;
-  gint                 argc;
-  gint                 n;
+  GSList  *result = NULL;
+  GSList  *lp;
+  gchar  **oargv;
+  gchar  **argv;
+  gint     argc;
+  gint     n;
 
   for (lp = app->windows; lp != NULL; lp = lp->next)
     {
       if (lp != app->windows)
-        result = g_list_append (result, g_strdup ("--window"));
-      result = g_list_concat (result, terminal_window_get_restart_command (lp->data));
+        result = g_slist_append (result, g_strdup ("--window"));
+      result = g_slist_concat (result, terminal_window_get_restart_command (lp->data));
     }
 
-  argc = g_list_length (result) + 1;
+  argc = g_slist_length (result) + 1;
   argv = g_new (gchar*, argc + 1);
   for (lp = result, n = 1; n < argc; lp = lp->next, ++n)
     argv[n] = lp->data;
@@ -309,7 +350,7 @@ terminal_app_save_yourself (ExoXsessionClient *client,
 
   if (exo_xsession_client_get_restart_command (client, &oargv, NULL))
     {
-      g_assert (oargv[0] != NULL);
+      terminal_assert (oargv[0] != NULL);
 
       argv[0] = oargv[0];
 
@@ -324,7 +365,7 @@ terminal_app_save_yourself (ExoXsessionClient *client,
 
   exo_xsession_client_set_restart_command (client, argv, argc);
 
-  g_list_free (result);
+  g_slist_free (result);
   g_strfreev (argv);
 }
 
@@ -399,19 +440,6 @@ terminal_app_find_screen (const gchar *display_name)
 
 
 
-/**
- * terminal_app_new:
- *
- * Return value :
- **/
-TerminalApp*
-terminal_app_new (void)
-{
-  return g_object_new (TERMINAL_TYPE_APP, NULL);
-}
-
-
-
 /**
  * terminal_app_process:
  * @app
@@ -427,8 +455,7 @@ terminal_app_process (TerminalApp  *app,
                       gint          argc,
                       GError      **error)
 {
-  GList  *attrs;
-  GList  *lp;
+  GSList *attrs, *lp;
 
   if (!terminal_options_parse (argc, argv, &attrs, NULL, error))
     return FALSE;
@@ -436,8 +463,8 @@ terminal_app_process (TerminalApp  *app,
   for (lp = attrs; lp != NULL; lp = lp->next)
     terminal_app_open_window (app, lp->data);
 
-  g_list_foreach (attrs, (GFunc) terminal_window_attr_free, NULL);
-  g_list_free (attrs);
+  g_slist_foreach (attrs, (GFunc) terminal_window_attr_free, NULL);
+  g_slist_free (attrs);
 
   return TRUE;
 }
@@ -459,10 +486,11 @@ terminal_app_open_window (TerminalApp         *app,
   GtkWidget       *window;
   GtkWidget       *terminal;
   GdkScreen       *screen;
-  GList           *lp;
+  gchar           *geometry;
+  GSList          *lp;
 
-  g_return_if_fail (TERMINAL_IS_APP (app));
-  g_return_if_fail (attr != NULL);
+  terminal_return_if_fail (TERMINAL_IS_APP (app));
+  terminal_return_if_fail (attr != NULL);
 
   window = terminal_app_create_window (app,
                                        attr->fullscreen,
@@ -473,7 +501,17 @@ terminal_app_open_window (TerminalApp         *app,
   if (attr->role != NULL)
     gtk_window_set_role (GTK_WINDOW (window), attr->role);
   if (attr->startup_id != NULL)
-    terminal_window_set_startup_id (TERMINAL_WINDOW (window), attr->startup_id);
+    gtk_window_set_startup_id (GTK_WINDOW (window), attr->startup_id);
+  if (attr->maximize)
+    gtk_window_maximize (GTK_WINDOW (window));
+
+  if (attr->icon != NULL)
+    {
+      if (g_path_is_absolute (attr->icon))
+        gtk_window_set_icon_from_file (GTK_WINDOW (window), attr->icon, NULL);
+      else
+        gtk_window_set_icon_name (GTK_WINDOW (window), attr->icon);
+    }
 
   screen = terminal_app_find_screen (attr->display);
   if (G_LIKELY (screen != NULL))
@@ -484,7 +522,7 @@ terminal_app_open_window (TerminalApp         *app,
 
   for (lp = attr->tabs; lp != NULL; lp = lp->next)
     {
-      terminal = terminal_screen_new ();
+      terminal = g_object_new (TERMINAL_TYPE_SCREEN, NULL);
 
       tab_attr = lp->data;
       if (tab_attr->command != NULL)
@@ -493,23 +531,31 @@ terminal_app_open_window (TerminalApp         *app,
         terminal_screen_set_working_directory (TERMINAL_SCREEN (terminal), tab_attr->directory);
       if (tab_attr->title != NULL)
         terminal_screen_set_custom_title (TERMINAL_SCREEN (terminal), tab_attr->title);
+      terminal_screen_set_hold (TERMINAL_SCREEN (terminal), tab_attr->hold);
 
       terminal_window_add (TERMINAL_WINDOW (window), TERMINAL_SCREEN (terminal));
 
-      /* if this was the first tab, we set the geometry string now
-       * and show the window. This is required to work around a hang
-       * in Gdk, which I failed to figure the cause til now.
-       */
-      if (lp == attr->tabs)
-        {
-          if (attr->geometry != NULL && !gtk_window_parse_geometry (GTK_WINDOW (window), attr->geometry))
-            g_printerr (_("Invalid geometry string \"%s\"\n"), attr->geometry);
-          gtk_widget_show (window);
-        }
-
       terminal_screen_launch_child (TERMINAL_SCREEN (terminal));
     }
 
+  /* set the window geometry, this can only be set after one of the tabs
+   * has been added, because vte is the geometry widget, so atleast one
+   * call should have been made to terminal_screen_set_window_geometry_hints */
+  if (G_LIKELY (attr->geometry == NULL))
+    g_object_get (G_OBJECT (app->preferences), "misc-default-geometry", &geometry, NULL);
+  else
+    geometry = g_strdup (attr->geometry);
+
+  /* try to apply the geometry to the window */
+  if (!gtk_window_parse_geometry (GTK_WINDOW (window), geometry))
+    g_printerr (_("Invalid geometry string \"%s\"\n"), geometry);
+
+  /* cleanup */
+  g_free (geometry);
+
+  /* show the window */
+  gtk_widget_show (window);
+
   /* register with session manager on first display
    * opened by Terminal. This is to ensure that we
    * get restarted by only _one_ session manager (the
@@ -524,7 +570,3 @@ terminal_app_open_window (TerminalApp         *app,
                         G_CALLBACK (terminal_app_save_yourself), app);
     }
 }
-
-
-
-
