@@ -26,12 +26,12 @@
 
 #include <terminal/terminal-options.h>
 
-G_BEGIN_DECLS;
+G_BEGIN_DECLS
 
 #define TERMINAL_ERROR (terminal_error_quark ())
 GQuark terminal_error_quark (void) G_GNUC_CONST;
 
-typedef enum /*< enum,prefix=TERMINAL_ERROR >*/
+enum _TerminalError
 {
   /* problem with the runtime linker */
   TERMINAL_ERROR_LINKER_FAILURE,
@@ -39,7 +39,7 @@ typedef enum /*< enum,prefix=TERMINAL_ERROR >*/
   TERMINAL_ERROR_USER_MISMATCH,
   /* general failure */
   TERMINAL_ERROR_FAILED,
-} TerminalError;
+};
 
 
 #define TERMINAL_TYPE_APP         (terminal_app_get_type ())
@@ -49,11 +49,10 @@ typedef enum /*< enum,prefix=TERMINAL_ERROR >*/
 
 typedef struct _TerminalAppClass TerminalAppClass;
 typedef struct _TerminalApp      TerminalApp;
+typedef enum   _TerminalError    TerminalError;
 
 GType        terminal_app_get_type            (void) G_GNUC_CONST;
 
-TerminalApp *terminal_app_new                 (void);
-
 gboolean     terminal_app_process             (TerminalApp        *app,
                                                gchar             **argv,
                                                gint                argc,
@@ -62,6 +61,6 @@ gboolean     terminal_app_process             (TerminalApp        *app,
 void         terminal_app_open_window         (TerminalApp        *app,
                                                TerminalWindowAttr *attr);
 
-G_END_DECLS;
+G_END_DECLS
 
 #endif /* !__TERMINAL_APP_H__ */
