@@ -1,4 +1,4 @@
-/* $Id: terminal-dbus.h 124 2005-01-31 20:41:46Z bmeurer $ */
+/* $Id$ */
 /*-
  * Copyright (c) 2004-2005 os-cillation e.K.
  *
@@ -24,7 +24,7 @@
 
 #include <terminal/terminal-app.h>
 
-G_BEGIN_DECLS;
+G_BEGIN_DECLS
 
 gboolean  terminal_dbus_register_service  (TerminalApp  *app,
                                            GError      **error);
@@ -32,6 +32,6 @@ gboolean  terminal_dbus_invoke_launch     (gint          argc,
                                            gchar       **argv,
                                            GError      **error);
 
-G_END_DECLS;
+G_END_DECLS
 
 #endif /* !__TERMINAL_DBUS_H__ */
