@@ -1,4 +1,4 @@
-/* $Id: terminal-widget.c 26427 2007-12-02 14:21:23Z benny $ */
+/* $Id$ */
 /*-
  * Copyright (c) 2004-2007 os-cillation e.K.
  *
@@ -37,22 +37,21 @@
 #include <terminal/terminal-marshal.h>
 #include <terminal/terminal-preferences.h>
 #include <terminal/terminal-widget.h>
+#include <terminal/terminal-private.h>
 
 
 
-#define USERCHARS       "-A-Za-z0-9"
-#define PASSCHARS       "-A-Za-z0-9,?;.:/!%$^*&~\"#'"
-#define HOSTCHARS       "-A-Za-z0-9"
-#define USER            "[" USERCHARS "]+(:["PASSCHARS "]+)?"
-#define MATCH_BROWSER1  "((file|https?|ftps?)://(" USER "@)?)[" HOSTCHARS ".]+(:[0-9]+)?" \
-                        "(/[-A-Za-z0-9_$.+!*(),;:@&=?/~#%]*[^]'.}>) \t\r\n,\\\"])?"
-#define MATCH_BROWSER2  "(www|ftp)[" HOSTCHARS "]*\\.[" HOSTCHARS ".]+(:[0-9]+)?" \
-                        "(/[-A-Za-z0-9_$.+!*(),;:@&=?/~#%]*[^]'.}>) \t\r\n,\\\"])?"
-#if !defined(__GLIBC__)
-#define MATCH_MAILER    "(mailto:)?[a-z0-9][a-z0-9.-]*@[a-z0-9][a-z0-9-]*(\\.[a-z0-9][a-z0-9-]*)+"
-#else
-#define MATCH_MAILER    "\\<(mailto:)?[a-z0-9][a-z0-9.-]*@[a-z0-9][a-z0-9-]*(\\.[a-z0-9][a-z0-9-]*)+\\>"
-#endif
+#define USERCHARS       "-[:alnum:]"
+#define USERCHARS_CLASS "[" USERCHARS "]"
+#define PASSCHARS_CLASS "[-[:alnum:]\\Q,?;.:/!%$^*&~\"#'\\E]"
+#define HOSTCHARS_CLASS "[-[:alnum:]]"
+#define HOST            HOSTCHARS_CLASS "+(\\." HOSTCHARS_CLASS "+)*"
+#define PORT            "(?:\\:[[:digit:]]{1,5})?"
+#define PATHCHARS_CLASS "[-[:alnum:]\\Q_$.+!*,;@&=?/~#'%\\E]"
+#define PATHTERM_CLASS  "[^\\Q]'.}>) \t\r\n,\"\\E]"
+#define SCHEME          "(?:news:|telnet:|nntp:|file:\\/|https?:|ftps?:|webcal:)"
+#define USERPASS        USERCHARS_CLASS "+(?:" PASSCHARS_CLASS "+)?"
+#define URLPATH         "(?:(/"PATHCHARS_CLASS"+(?:[(]"PATHCHARS_CLASS"*[)])*"PATHCHARS_CLASS"*)*"PATHTERM_CLASS")?"
 
 
 
@@ -64,21 +63,29 @@ enum
 
 enum
 {
-  TARGET_URI_LIST,
-  TARGET_UTF8_STRING,
-  TARGET_TEXT,
-  TARGET_COMPOUND_TEXT,
-  TARGET_STRING,
-  TARGET_TEXT_PLAIN,
-  TARGET_MOZ_URL,
-  TARGET_APPLICATION_X_COLOR,
-  TARGET_GTK_NOTEBOOK_TAB,
+  PATTERN_TYPE_NONE,
+  PATTERN_TYPE_FULL_HTTP,
+  PATTERN_TYPE_HTTP,
+  PATTERN_TYPE_EMAIL
+};
+
+typedef struct
+{
+  const gchar *pattern;
+  gint         type;
+}
+TerminalRegexPattern;
+
+static const TerminalRegexPattern regex_patterns[] =
+{
+  { SCHEME "//(?:" USERPASS "\\@)?" HOST PORT URLPATH, PATTERN_TYPE_FULL_HTTP },
+  { "(?:www|ftp)" HOSTCHARS_CLASS "*\\." HOST PORT URLPATH, PATTERN_TYPE_HTTP },
+  { "(?:mailto:)?" USERCHARS_CLASS "[" USERCHARS ".]*\\@" HOSTCHARS_CLASS "+\\." HOST, PATTERN_TYPE_EMAIL },
+  { "news:[[:alnum:]\\Q^_{|}~!\"#$%&'()*+,./;:=?`\\E]+", PATTERN_TYPE_FULL_HTTP }
 };
 
 
 
-static void     terminal_widget_class_init            (TerminalWidgetClass  *klass);
-static void     terminal_widget_init                  (TerminalWidget       *widget);
 static void     terminal_widget_finalize              (GObject              *object);
 static gboolean terminal_widget_button_press_event    (GtkWidget            *widget,
                                                        GdkEventButton       *event);
@@ -112,9 +119,7 @@ struct _TerminalWidget
 
   /*< private >*/
   TerminalPreferences *preferences;
-  gint                 tag_browser1;
-  gint                 tag_browser2;
-  gint                 tag_mailer;
+  gint                *regex_tags;
 };
 
 
@@ -138,7 +143,7 @@ static const GtkTargetEntry targets[] =
 
 
 
-G_DEFINE_TYPE (TerminalWidget, terminal_widget, VTE_TYPE_TERMINAL);
+G_DEFINE_TYPE (TerminalWidget, terminal_widget, VTE_TYPE_TERMINAL)
 
 
 
@@ -163,8 +168,7 @@ terminal_widget_class_init (TerminalWidgetClass *klass)
     g_signal_new (I_("context-menu"),
                   G_TYPE_FROM_CLASS (klass),
                   G_SIGNAL_RUN_LAST,
-                  G_STRUCT_OFFSET (TerminalWidgetClass, get_context_menu),
-                  NULL, NULL,
+                  0, NULL, NULL,
                   _terminal_marshal_OBJECT__VOID,
                   GTK_TYPE_MENU, 0);
 }
@@ -174,9 +178,12 @@ terminal_widget_class_init (TerminalWidgetClass *klass)
 static void
 terminal_widget_init (TerminalWidget *widget)
 {
-  widget->tag_browser1 = -1;
-  widget->tag_browser2 = -1;
-  widget->tag_mailer   = -1;
+  guint i;
+
+  /* initialize regex tag ids */
+  widget->regex_tags = g_new (gint, G_N_ELEMENTS (regex_patterns));
+  for (i = 0; i < G_N_ELEMENTS (regex_patterns); i++)
+    widget->regex_tags[i] = -1;
 
   /* query preferences connection */
   widget->preferences = terminal_preferences_get ();
@@ -204,6 +211,9 @@ terminal_widget_finalize (GObject *object)
 {
   TerminalWidget *widget = TERMINAL_WIDGET (object);
 
+  /* free tag ids */
+  g_free (widget->regex_tags);
+
   /* disconnect the misc-highlight-urls watch */
   g_signal_handlers_disconnect_by_func (G_OBJECT (widget->preferences), G_CALLBACK (terminal_widget_update_highlight_urls), widget);
 
@@ -259,7 +269,7 @@ terminal_widget_context_menu_open (TerminalWidget *widget,
 static void
 terminal_widget_context_menu (TerminalWidget *widget,
                               gint            button,
-                              gint            time,
+                              guint32         event_time,
                               gint            x,
                               gint            y)
 {
@@ -271,8 +281,9 @@ terminal_widget_context_menu (TerminalWidget *widget,
   GtkWidget   *item_separator = NULL;
   GList       *children;
   gchar       *match;
-  guint        id;
+  guint        id, i;
   gint         tag;
+  gint         pattern_type = PATTERN_TYPE_NONE;
 
   g_signal_emit (G_OBJECT (widget), widget_signals[GET_CONTEXT_MENU], 0, &menu);
   if (G_UNLIKELY (menu == NULL))
@@ -295,8 +306,17 @@ terminal_widget_context_menu (TerminalWidget *widget,
         item_separator = NULL;
       g_list_free (children);
 
+      /* lookup the pattern type */
+      for (i = 0; i < G_N_ELEMENTS (regex_patterns); i++)
+        if (widget->regex_tags[i] == tag)
+          {
+            pattern_type = regex_patterns[i].type;
+            break;
+          }
+      terminal_return_if_fail (pattern_type != PATTERN_TYPE_NONE);
+
       /* create menu items with appriorate labels */
-      if (tag == widget->tag_mailer)
+      if (pattern_type == PATTERN_TYPE_EMAIL)
         {
           item_copy = gtk_menu_item_new_with_label (_("Copy Email Address"));
           item_open = gtk_menu_item_new_with_label (_("Compose Email"));
@@ -324,11 +344,10 @@ terminal_widget_context_menu (TerminalWidget *widget,
     }
 
   /* take a reference on the menu */
-  g_object_ref (G_OBJECT (menu));
-  gtk_object_sink (GTK_OBJECT (menu));
+  g_object_ref_sink (G_OBJECT (menu));
 
-  if (time < 0)
-    time = gtk_get_current_event_time ();
+  if (event_time == 0)
+    event_time = gtk_get_current_event_time ();
 
   loop = g_main_loop_new (NULL, FALSE);
 
@@ -340,7 +359,7 @@ terminal_widget_context_menu (TerminalWidget *widget,
 
   /* run our custom main loop */
   gtk_grab_add (menu);
-  gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, button, time);
+  gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, button, event_time);
   g_main_loop_run (loop);
   g_main_loop_unref (loop);
   gtk_grab_remove (menu);
@@ -428,7 +447,7 @@ terminal_widget_drag_data_received (GtkWidget        *widget,
                                     gint              y,
                                     GtkSelectionData *selection_data,
                                     guint             info,
-                                    guint             time)
+                                    guint32           drag_time)
 {
   const guint16 *ucs;
   GdkColor       color;
@@ -450,7 +469,7 @@ terminal_widget_drag_data_received (GtkWidget        *widget,
       text = (gchar *) gtk_selection_data_get_text (selection_data);
       if (G_LIKELY (text != NULL))
         {
-          if (G_LIKELY (*text != '\0'))
+          if (G_LIKELY (IS_STRING (text)))
             vte_terminal_feed_child (VTE_TERMINAL (widget), text, strlen (text));
           g_free (text);
         }
@@ -574,7 +593,7 @@ terminal_widget_drag_data_received (GtkWidget        *widget,
       if (G_LIKELY (screen))
         {
           g_signal_emit_by_name (G_OBJECT (screen), "drag-data-received", context,
-                                 x, y, selection_data, info, time);
+                                 x, y, selection_data, info, drag_time);
         }
       break;
 
@@ -584,7 +603,7 @@ terminal_widget_drag_data_received (GtkWidget        *widget,
     }
 
   if (info != TARGET_GTK_NOTEBOOK_TAB)
-    gtk_drag_finish (context, TRUE, FALSE, time);
+    gtk_drag_finish (context, TRUE, FALSE, drag_time);
 }
 
 
@@ -641,40 +660,61 @@ terminal_widget_open_uri (TerminalWidget *widget,
                           const gchar    *link,
                           gint            tag)
 {
-  GError *error = NULL;
-  gchar  *uri;
+  GError    *error = NULL;
+  gchar     *uri;
+  guint      i;
+  GdkScreen *screen;
 
-  /* parse the URI from the link */
-  if (tag == widget->tag_browser1)
+  for (i = 0; i < G_N_ELEMENTS (regex_patterns); i++)
     {
-      uri = g_strdup (link);
-    }
-  else if (tag == widget->tag_browser2)
-    {
-      uri = g_strconcat ("http://", link, NULL);
-    }
-  else if (tag == widget->tag_mailer)
-    {
-      if (strncmp (link, "mailto:", 7) == 0)
-        uri = g_strdup (link);
-      else
-        uri = g_strconcat ("mailto:", link, NULL);
-    }
-  else
-    {
-      g_warning ("Invalid tag specified while trying to open an URI.");
+      /* lookup the tag in our tags */
+      if (widget->regex_tags[i] != tag)
+        continue;
+
+      /* handle the pattern type */
+      switch (regex_patterns[i].type)
+        {
+          case PATTERN_TYPE_FULL_HTTP:
+            uri = g_strdup (link);
+            break;
+
+          case PATTERN_TYPE_HTTP:
+            uri = g_strconcat ("http://", link, NULL);
+            break;
+
+          case PATTERN_TYPE_EMAIL:
+            if (strncmp (link, "mailto:", 7) == 0)
+              uri = g_strdup (link);
+            else
+              uri = g_strconcat ("mailto:", link, NULL);
+            break;
+
+          default:
+            goto invalid_tag;
+        }
+
+      /* try to open the URI with the responsible application */
+      screen = gtk_widget_get_screen (GTK_WIDGET (widget));
+#if EXO_CHECK_VERSION (0, 5, 0)
+      if (!gtk_show_uri (screen, uri, gtk_get_current_event_time (), &error))
+#else
+      if (!exo_url_show_on_screen (uri, NULL, screen, &error))
+#endif
+        {
+          /* tell the user that we were unable to open the responsible application */
+          terminal_dialogs_show_error (widget, error, _("Failed to open the URL `%s'"), uri);
+          g_error_free (error);
+        }
+
+      g_free (uri);
+
+      /* done */
       return;
     }
 
-  /* try to open the URI with the responsible application */
-  if (!exo_url_show_on_screen (uri, NULL, gtk_widget_get_screen (GTK_WIDGET (widget)), &error))
-    {
-      /* tell the user that we were unable to open the responsible application */
-      terminal_dialogs_show_error (widget, error, _("Failed to open the URL `%s'"), uri);
-      g_error_free (error);
-    }
+invalid_tag:
 
-  g_free (uri);
+  g_warning ("Invalid tag specified while trying to open link \"%s\".", link);
 }
 
 
@@ -682,69 +722,58 @@ terminal_widget_open_uri (TerminalWidget *widget,
 static void
 terminal_widget_update_highlight_urls (TerminalWidget *widget)
 {
-  gboolean highlight_urls;
+  guint                       i;
+  gboolean                    highlight_urls;
+  GRegex                     *regex;
+  const TerminalRegexPattern *pattern;
+  GError                     *error;
+
+  g_object_get (G_OBJECT (widget->preferences),
+                "misc-highlight-urls", &highlight_urls, NULL);
 
-  g_object_get (G_OBJECT (widget->preferences), "misc-highlight-urls", &highlight_urls, NULL);
   if (!highlight_urls)
     {
-      if (widget->tag_browser1 >= 0)
-        {
-          vte_terminal_match_remove (VTE_TERMINAL (widget), widget->tag_browser1);
-          widget->tag_browser1 = -1;
-        }
-
-      if (widget->tag_browser2 >= 0)
-        {
-          vte_terminal_match_remove (VTE_TERMINAL (widget), widget->tag_browser2);
-          widget->tag_browser2 = -1;
-        }
-
-      if (widget->tag_mailer >= 0)
-        {
-          vte_terminal_match_remove (VTE_TERMINAL (widget), widget->tag_mailer);
-          widget->tag_mailer = -1;
-        }
+      /* remove all our regex tags */
+      for (i = 0; i < G_N_ELEMENTS (regex_patterns); i++)
+        if (widget->regex_tags[i] != -1)
+          {
+            vte_terminal_match_remove (VTE_TERMINAL (widget),
+                                       widget->regex_tags[i]);
+            widget->regex_tags[i] = -1;
+          }
     }
   else
     {
-      if (widget->tag_browser1 < 0)
+      /* set all our patterns */
+      for (i = 0; i < G_N_ELEMENTS (regex_patterns); i++)
         {
-          widget->tag_browser1 = vte_terminal_match_add (VTE_TERMINAL (widget),
-                                                         MATCH_BROWSER1);
-          vte_terminal_match_set_cursor_type (VTE_TERMINAL (widget),
-                                              widget->tag_browser1,
-                                              GDK_HAND2);
-        }
+          /* continue if already set */
+          if (G_UNLIKELY (widget->regex_tags[i] != -1))
+            continue;
 
-      if (widget->tag_browser2 < 0)
-        {
-          widget->tag_browser2 = vte_terminal_match_add (VTE_TERMINAL (widget),
-                                                         MATCH_BROWSER2);
-          vte_terminal_match_set_cursor_type (VTE_TERMINAL (widget),
-                                              widget->tag_browser2,
-                                              GDK_HAND2);
-        }
+          /* get the pattern */
+          pattern = &regex_patterns[i];
 
-      if (widget->tag_mailer < 0)
-        {
-          widget->tag_mailer = vte_terminal_match_add (VTE_TERMINAL (widget),
-                                                       MATCH_MAILER);
+          /* build the regex */
+          error = NULL;
+          regex = g_regex_new (pattern->pattern,
+                               G_REGEX_CASELESS | G_REGEX_OPTIMIZE,
+                               0, &error);
+          if (G_UNLIKELY (error != NULL))
+            {
+              g_critical ("Failed to parse regular expression pattern %d: %s",
+                          i, error->message);
+              g_error_free (error);
+              continue;
+            }
+
+          /* set the new regular expression */
+          widget->regex_tags[i] = vte_terminal_match_add_gregex (VTE_TERMINAL (widget),
+                                                                 regex, 0);
           vte_terminal_match_set_cursor_type (VTE_TERMINAL (widget),
-                                              widget->tag_mailer,
-                                              GDK_HAND2);
+                                              widget->regex_tags[i], GDK_HAND2);
+          /* release the regex owned by vte now */
+          g_regex_unref (regex);
         }
     }
 }
-
-
-
-/**
- * terminal_widget_new:
- *
- * Return value:
- **/
-GtkWidget*
-terminal_widget_new (void)
-{
-  return g_object_new (TERMINAL_TYPE_WIDGET, NULL);
-}
